using System.Web.Mvc;
using Microsoft.Practices.Unity;
using DAL = Scrum.Web.Models.DAL;
using UI = Scrum.Web.Models.UI;
using Unity.Mvc4;
using AutoMapper;
using System.Linq;
using System;
using System.Collections.Generic;
using PagedList;

namespace Scrum.Web
{
    public static class Bootstrapper
    {
        public static IUnityContainer Initialise()
        {
            var container = BuildUnityContainer();

            DependencyResolver.SetResolver(new UnityDependencyResolver(container));

            RegisterAutoMapper();

            return container;
        }

        private static IUnityContainer BuildUnityContainer()
        {
            var container = new UnityContainer();

            // register all your components with the container here
            // it is NOT necessary to register your controllers

            // e.g. container.RegisterType<ITestService, TestService>();
            RegisterTypes(container);

            return container;
        }

        public static void RegisterTypes(IUnityContainer container)
        {
            container.RegisterType<DAL.IRepository, DAL.Repository>();
            container.RegisterType<Scrum.Web.Helpers.IScrumMembershipProvider, Scrum.Web.Helpers.ScrumMembershipProvider>();
            container.RegisterType<Scrum.Web.Helpers.IScrumRoleProvider, Scrum.Web.Helpers.ScrumRoleProvider>();
        }

        public static void RegisterAutoMapper()
        {
            Mapper.CreateMap<DAL.User, Scrum.Web.Models.PersonalDataEditModel>()
                .ForMember(dto => dto.NewPassword, opt => opt.Ignore())
                .ForMember(dto => dto.ConfirmPassword, opt => opt.Ignore());

            // ProjectController
            Mapper.CreateMap<DAL.Project, UI.ProjectListModel>();
            Mapper.CreateMap<DAL.Project, UI.ProjectEditModel>();
            Mapper.CreateMap<DAL.Project, UI.ProjectDetailModel>();
            Mapper.CreateMap<DAL.Project, UI.ProjectRoleListModel>()
                .ForMember(dto => dto.Owners, opt => opt.ResolveUsing(obj => obj.Permission.Where(cond => cond.ProfileId == 1).Select(us => us.User )))
                .ForMember(dto => dto.ScrumMasters, opt => opt.ResolveUsing(obj => obj.Permission.Where(cond => cond.ProfileId == 2).Select(us => us.User)))
                .ForMember(dto => dto.Developers, opt => opt.ResolveUsing(obj => obj.Permission.Where(cond => cond.ProfileId == 3).Select(us => us.User)));
            Mapper.CreateMap<DAL.User, UI.ProjectRoleListModelUser>();
            Mapper.CreateMap<Tuple<DAL.Project, DAL.Profile, IEnumerable<DAL.User>>, UI.ProjectCreateRoleModel>()
                .ForMember(dto => dto.ProjectId, opt => opt.ResolveUsing(obj => obj.Item1.ProjectId))
                .ForMember(dto => dto.ProjectShortName, opt => opt.ResolveUsing(obj => obj.Item1.ShortName))
                .ForMember(dto => dto.ProjectName, opt => opt.ResolveUsing(obj => obj.Item1.Name))
                .ForMember(dto => dto.ProfileId, opt => opt.ResolveUsing(obj => obj.Item2.ProfileId))
                .ForMember(dto => dto.ProfileName, opt => opt.ResolveUsing(obj => obj.Item2.Name))
                .ForMember(dto => dto.Users, opt => opt.ResolveUsing(obj => obj.Item3))
                .ForMember(dto => dto.SelectedUserId, opt => opt.Ignore());
            
            // ProjecteSelectorController
            Mapper.CreateMap<IEnumerable<DAL.Project>, UI.ProjectSelectorModel>()
                .ForMember(dto => dto.Projects, opt => opt.ResolveUsing(obj => obj))
                .ForMember(dto => dto.SelectedProjects, opt => opt.Ignore());
            Mapper.CreateMap<DAL.Project, UI.ProjectSelectorProjectModel>();

            // UserController
            Mapper.CreateMap<DAL.User, UI.UserListModel>();
            Mapper.CreateMap<DAL.User, UI.UserEditModel>();
            Mapper.CreateMap<DAL.User, UI.UserDetailModel>();
            Mapper.CreateMap<DAL.User, UI.UserNewPasswordModel>()
                .ForMember(dto => dto.Password, opt => opt.Ignore())
                .ForMember(dto => dto.ConfirmPassword, opt => opt.Ignore());
            Mapper.CreateMap<DAL.User, UI.UserRoleListModel>()
               .ForMember(dto => dto.Owners, opt => opt.Ignore())
               .ForMember(dto => dto.ScrumMasters, opt => opt.Ignore())
               .ForMember(dto => dto.Developers, opt => opt.Ignore());
            
            // ProductBackLogController
            Mapper.CreateMap<DAL.Project, UI.ProductBackLogListModel>()
                .ForMember(dto => dto.UserHistories, opt => opt.ResolveUsing(obj => obj.UserHistory.Where(uh => uh.UserHistoryStatusId != (int)Config.UserHistoryStatus.Finnished).OrderBy(uh => uh.Priority)));
            Mapper.CreateMap<DAL.UserHistory, UI.ProductBackLogUserHistoryModel>()
                .ForMember(dto => dto.CanDelete, opt => opt.ResolveUsing(obj => obj.Task.Count == 0));
            Mapper.CreateMap<Tuple<DAL.Project, IQueryable<DAL.UserHistoryStatus>>, UI.ProductBackLogCreateModel>()
                .ForMember(dto => dto.ProjectId, opt => opt.ResolveUsing(obj => obj.Item1.ProjectId))
                .ForMember(dto => dto.ProjectName, opt => opt.ResolveUsing(obj => obj.Item1.Name))
                .ForMember(dto => dto.Name, opt => opt.Ignore())
                .ForMember(dto => dto.UserHistoryStatusId, opt => opt.Ignore())
                .ForMember(dto => dto.Priority, opt => opt.Ignore())
                .ForMember(dto => dto.Description, opt => opt.Ignore())
                .ForMember(dto => dto.Estimation, opt => opt.Ignore())
                .ForMember(dto => dto.UserHistoryStatus, opt => opt.ResolveUsing(obj => obj.Item2));
            Mapper.CreateMap<DAL.UserHistory, UI.ProductBackLogEditModel>();
            Mapper.CreateMap<DAL.UserHistoryStatus, UI.ProductBackLogUserHistoryStatusModel>();

            // SprintController
            Mapper.CreateMap<DAL.Project, UI.SprintListModel>()
                .ForMember(dto => dto.CanCreateSprint, opt => opt.Ignore())
                .ForMember(dto => dto.CanFinnishSprint, opt => opt.Ignore())
                .ForMember(dto => dto.CanAssignOrFinnishTask, opt => opt.Ignore())
                .ForMember(dto => dto.BeginDate, opt => opt.ResolveUsing(obj => obj.Sprint.Any(spr => spr.BeginDate <= DateTime.Today && DateTime.Today <= spr.FinnishDate) ? obj.Sprint.SingleOrDefault(spr => spr.BeginDate <= DateTime.Today && DateTime.Today <= spr.FinnishDate).BeginDate : new Nullable<DateTime>()))
                .ForMember(dto => dto.FinnishDate, opt => opt.ResolveUsing(obj => obj.Sprint.Any(spr => spr.BeginDate <= DateTime.Today && DateTime.Today <= spr.FinnishDate) ? obj.Sprint.SingleOrDefault(spr => spr.BeginDate <= DateTime.Today && DateTime.Today <= spr.FinnishDate).FinnishDate : new Nullable<DateTime>()))
                .ForMember(dto => dto.SprintId, opt => opt.ResolveUsing(obj => obj.Sprint.Any(spr => spr.BeginDate <= DateTime.Today && DateTime.Today <= spr.FinnishDate) ? obj.Sprint.SingleOrDefault(spr => spr.BeginDate <= DateTime.Today && DateTime.Today <= spr.FinnishDate).SprintId : 0))
                .ForMember(dto => dto.Tasks, opt => opt.ResolveUsing(obj => obj.UserHistory
                   .Where(usr => usr.SprintBackLogUserHistory.Any(sprbl => sprbl.SprintId == obj.Sprint.Where(spr => spr.BeginDate.Date <= DateTime.Today && DateTime.Today <= spr.FinnishDate.Date).DefaultIfEmpty(new DAL.Sprint()).Single().SprintId))
                   .OrderBy(uh => uh.Priority).SelectMany(uh => uh.Task)));
            Mapper.CreateMap<DAL.Project, UI.SprintCreateModel>()
                .ForMember(dto => dto.BeginDate, opt => opt.Ignore())
                .ForMember(dto => dto.FinnishDate, opt => opt.Ignore())
                .ForMember(dto => dto.UserHistories, opt => opt.ResolveUsing(obj => obj.UserHistory.Where(uh => uh.UserHistoryStatusId != (int)Config.UserHistoryStatus.Finnished).OrderBy(uh => uh.Priority)));
            Mapper.CreateMap<DAL.UserHistory, UI.SprintUserHistoryModel>()
                .ForMember(dto => dto.Selected, opt => opt.Ignore())
                .ForMember(dto => dto.TotalTaskEstimation, opt => opt.ResolveUsing(obj => obj.Task.Sum(task => task.Estimation)));
            Mapper.CreateMap<DAL.Task, UI.SprintTaskModel>()
                .ForMember(dto => dto.ProjectId, opt => opt.ResolveUsing(obj => obj.UserHistory.ProjectId))
                .ForMember(dto => dto.Developers, opt => opt.Ignore())
                .ForMember(dto => dto.TaskStatus, opt => opt.ResolveUsing(obj => obj.TaskStatus.Name))
                .ForMember(dto => dto.TotalWorkLog, opt => opt.ResolveUsing(obj => obj.WorkLog.Sum(wl => wl.TimeSpent)))
                .ForMember(dto => dto.AssignedUserName, opt => opt.ResolveUsing(obj => obj.User != null ? obj.User.Name : string.Empty));
            Mapper.CreateMap<DAL.UserHistory, UI.SprintEditModel>()
                .ForMember(dto => dto.Tasks, opt => opt.ResolveUsing(obj => obj.Task));
            Mapper.CreateMap<DAL.UserHistory, UI.SprintTaskModel>()
                .ForMember(dto => dto.TaskId, opt => opt.Ignore())
                .ForMember(dto => dto.TaskStatusId, opt => opt.Ignore())
                .ForMember(dto => dto.TaskStatus, opt => opt.Ignore())
                .ForMember(dto => dto.AssignedUserId, opt => opt.Ignore())
                .ForMember(dto => dto.UserHistoryName, opt => opt.ResolveUsing(obj => obj.Name))
                .ForMember(dto => dto.Name, opt => opt.Ignore())
                .ForMember(dto => dto.Description, opt => opt.Ignore())
                .ForMember(dto => dto.Developers, opt => opt.Ignore())
                .ForMember(dto => dto.TotalWorkLog, opt => opt.Ignore())
                .ForMember(dto => dto.AssignedUserName, opt => opt.Ignore());
            Mapper.CreateMap<DAL.User, UI.SprintUserModel>();
            Mapper.CreateMap<DAL.Task, UI.SprintAssignTaskModel>()
                .ForMember(dto => dto.ProjectId, opt => opt.ResolveUsing(obj => obj.UserHistory.ProjectId))
                .ForMember(dto => dto.Developers, opt => opt.Ignore())
                .ForMember(dto => dto.SprintId, opt => opt.Ignore());
            Mapper.CreateMap<DAL.Task, UI.SprintFinnishTaskModel>()
               .ForMember(dto => dto.ProjectId, opt => opt.ResolveUsing(obj => obj.UserHistory.ProjectId))
               .ForMember(dto => dto.AssignedUserName, opt => opt.ResolveUsing(obj => obj.User.Name))
               .ForMember(dto => dto.WorkLogItem, opt => opt.Ignore())
               .ForMember(dto => dto.SprintId, opt => opt.Ignore());
             
            // StatsController
            Mapper.CreateMap<DAL.Project, UI.StatsListModel>()
                .ForMember(dto => dto.Sprints, opt => opt.ResolveUsing(obj => obj.Sprint));
            Mapper.CreateMap<DAL.Sprint, UI.StatsSprintModel>()
                .ForMember(dto => dto.UserHistories, opt => opt.ResolveUsing(obj => obj.SprintBackLogUserHistory.Select(us => us.UserHistory)))
                .ForMember(dto => dto.Chart, opt => opt.Ignore());
            Mapper.CreateMap<DAL.UserHistory, UI.StatsUserHistoryModel>()
                .ForMember(dto => dto.TotalTaskEstimation, opt => opt.ResolveUsing(obj => obj.Task.Sum(t => t.Estimation)))
                .ForMember(dto => dto.NumberOfTasks, opt => opt.ResolveUsing(obj => obj.Task.Count));
            
        }
    }
}