﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;
using PagedList;

namespace Scrum.Web.Controllers
{
    [Authorize(Roles = "admin")]
    public class ProjectController : Controller
    {
        private readonly IRepository _repository;
        private readonly IScrumMembershipProvider _membershipService;
        private readonly IScrumRoleProvider _authorizationService;

        public ProjectController(IRepository repository, IScrumMembershipProvider membershipService, IScrumRoleProvider authorizationService)
        {
            this._repository = repository;
            this._membershipService = membershipService;
            this._authorizationService = authorizationService;
        }

        //
        // GET: /Project/

        public ActionResult Index(int? page)
        {
            var projects = _repository.All<Project>();
            var pageNumber = page ?? 1;

            var model = Mapper.Map<IEnumerable<ProjectListModel>>(projects).ToPagedList(pageNumber, Config.PageSize);

            return View(model);
        }

        //
        // GET: /Project/Details/5

        public ActionResult Details(int id = 0)
        {
            var project = this._repository.Single<Project>(id);
            if (project == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<ProjectDetailModel>(project);

            return View(model);
        }

        //
        // GET: /Project/Create

        public ActionResult Create()
        {
            return View();
        }

        //
        // POST: /Project/Create

        [HttpPost]
        public ActionResult Create(ProjectCreateModel model)
        {
            if (ModelState.IsValid)
            {
                Project project = new Project();
                project.ShortName = model.ShortName;
                project.Name = model.Name;
                project.CreatedDate = DateTime.Now;

                _repository.Add<Project>(project);
                _repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //
        // GET: /Project/Edit/5

        public ActionResult Edit(int id = 0)
        {
            var project = this._repository.Single<Project>(id);
            if (project == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<ProjectEditModel>(project);
            
            return View(model);
        }

        //
        // POST: /Project/Edit/5

        [HttpPost]
        public ActionResult Edit(ProjectEditModel model)
        {
            if (ModelState.IsValid)
            {
                var project = this._repository.Single<Project>(model.ProjectId);
                project.Name = model.Name;
                project.ShortName = model.ShortName;
                project.ModifiedDate = DateTime.Now;

                this._repository.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(model);
        }

        //
        // GET: /Project/Delete/5

        public ActionResult Delete(int id = 0)
        {
            var project = this._repository.Single<Project>(id);
            if (project == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<ProjectDetailModel>(project);

            return View(model);
        }

        //
        // POST: /Project/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            this._repository.Delete<Project>(id);
            this._repository.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            this._repository.Dispose();
            base.Dispose(disposing);
        }

         //
        // GET: /Project/ListRole/5

        public ActionResult ListRole(int id = 0)
        {
            var project = this._repository.Single<Project>(id);
            if (project == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<ProjectRoleListModel>(project);

            return View(model);
        }

        //
        // GET: /Project/DeleteRole/

        public ActionResult DeleteRole(int projectId = 0, int userId = 0, int profileId = 0)
        {
            this._repository.Delete<Permission>(per => per.ProjectId == projectId && per.UserId == userId && per.ProfileId == profileId);
            this._repository.SaveChanges();
            return RedirectToAction("ListRole", new { id = projectId });
        }

        //
        // GET: /Project/CreateRole

        public ActionResult CreateRole(int projectId = 0, int profileId = 0)
        {
            var project = this._repository.Single<Project>(projectId);
            var profile = this._repository.Single<Scrum.Web.Models.DAL.Profile>(profileId);
            //var usersWithProfileInProject = this._repository.Query<Permission>(per => per.ProjectId == projectId && per.ProfileId == profileId).ToList();
            //var users = this._repository.Query<User>(user => usersWithProfileInProject.Any(us => us.UserId == user.UserId)).AsEnumerable();
            var users = this._repository.All<User>()
                .Except(this._repository.Query<User>(user => user.Permission.Any(per => per.ProjectId == projectId && per.ProfileId == profileId)))
                .AsEnumerable();

            if (project == null || profile == null)
                return HttpNotFound();

            var model = Mapper.Map<ProjectCreateRoleModel>(Tuple.Create(project, profile, users));
            
            return View(model);
        }

        //
        // POST: /Project/CreateRole

        [HttpPost]
        public ActionResult CreateRole(ProjectCreateRoleModel model)
        {
            if (ModelState.IsValid)
            {
                var permission = new Permission();
                permission.ProjectId = model.ProjectId;
                permission.ProfileId = model.ProfileId;
                permission.UserId = model.SelectedUserId;

                _repository.Add<Permission>(permission);
                _repository.SaveChanges();

                return RedirectToAction("ListRole", new { id = model.ProjectId });
            }

            return View(model);
        }


        [ChildActionOnly]
        public PartialViewResult ProjectSelector()
        {
            var userId = (int)this._membershipService.GetUser().ProviderUserKey;
            var projects = this._repository.Query<Project>(pr => pr.Permission.Any(per => per.UserId == userId));

            var model = Mapper.Map<ProjectSelectorModel>(projects);
            model.SelectedProjects = new List<int>();
            model.SelectedProjects.Add(1);
            model.SelectedProjects.Add(5);
            model.SelectedProjects.Add(10);

            return PartialView(model);
        }
    }
}