﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;
using PagedList;

namespace Scrum.Web.Controllers
{
    public class SprintController : Controller
    {
 
        private readonly IRepository _repository;
        private readonly IScrumMembershipProvider _membershipService;
        private readonly IScrumRoleProvider _authorizationService;

        public SprintController(IRepository repository, IScrumMembershipProvider membershipService, IScrumRoleProvider authorizationService)
        {
            this._repository = repository;
            this._membershipService = membershipService;
            this._authorizationService = authorizationService;
        }

        //
        // GET: /Sprint/

        public ActionResult Index(int? projectId, int? page)
        {
            var selectedProjects = this._membershipService.GetSelectedProjects();
            var userId = (int)this._membershipService.GetUser().ProviderUserKey;
            var projects = this._repository.Query<Project>(proj => selectedProjects.Contains(proj.ProjectId));

            var model = Mapper.Map<IEnumerable<SprintListModel>>(projects);

            foreach (var item in model)
            {
                var pageNumber = 1;
                if (item.ProjectId == projectId)
                    pageNumber = page ?? 1;

                item.Tasks = item.Tasks.ToPagedList(pageNumber, Config.PageSize);
                item.CanAssignOrFinnishTask = this._repository.All<Permission>().Any(per => per.ProjectId == item.ProjectId
                    && per.ProfileId == (int)Config.Profile.Developer
                    && per.UserId == userId);
                item.CanCreateSprint = this._repository.All<Permission>().Any(per => per.ProjectId == item.ProjectId
                    && (per.ProfileId == (int)Config.Profile.ScrumMaster || per.ProfileId == (int)Config.Profile.Developer)
                    && per.UserId == userId);
                item.CanFinnishSprint = this._repository.All<Permission>().Any(per => per.ProjectId == item.ProjectId
                    && (per.ProfileId == (int)Config.Profile.ScrumMaster || per.ProfileId == (int)Config.Profile.ProductOwner)
                    && per.UserId == userId); 
            }

            return View(model);
        }

        //
        // GET: /Sprint/ManageTasks/5

        public ActionResult ManageTasks(long id = 0, int? projectId = 0)
        {
            var userHistory = this._repository.Single<UserHistory>(id);

            var model = Mapper.Map<SprintEditModel>(userHistory);

            return View(model);
        }

        //
        // GET: /Sprint/CreateTask/5

        public ActionResult CreateTask(long userHistoryid = 0)
        {
            var userHistory = this._repository.Single<UserHistory>(userHistoryid);

            var model = Mapper.Map<SprintTaskModel>(userHistory);

            return View(model);
        }

        //
        // POST: /Sprint/CreateTask/5

        [HttpPost]
        public ActionResult CreateTask(SprintTaskModel model)
        {
            if (ModelState.IsValid)
            {
                var task = new Task();
                task.UserHistoryId = model.UserHistoryId;
                task.TaskStatusId = (int)Config.TaskStatus.Pending;
                task.Description = model.Description;
                task.Estimation = model.Estimation;
                task.Name = model.Name;

                this._repository.Add<Task>(task);
                this._repository.SaveChanges();

                return RedirectToAction("ManageTasks", new { id = model.UserHistoryId, projectId = model.ProjectId});
            }

            return View(model); 
        }


        //
        // GET: /Sprint/Create

        public ActionResult Create(int? projectId)
        {
            var project = this._repository.Single<Project>(projectId);
            if (project == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<SprintCreateModel>(project);

            return View(model);
        }

        //
        // POST: /Sprint/Create

        [HttpPost]
        public ActionResult Create(SprintCreateModel model)
        {
            if (ModelState.IsValid)
            {
                var existUserHistoryWithouTasks = false;
                foreach (var item in model.UserHistories)
                {
                    if (item.Selected)
                    {
                        if (this._repository.Query<Task>(task => task.UserHistoryId == item.UserHistoryId).Count() == 0)
                            existUserHistoryWithouTasks = true;
                    }
                }

                if (existUserHistoryWithouTasks)
                {
                    ModelState.AddModelError(string.Empty, "Existeixen històries d'usuari que no tenen tasques assignades.");
                }
                else
                {
                    var sprint = new Sprint();
                    sprint.ProjectId = model.ProjectId;
                    sprint.BeginDate = model.BeginDate.Value;
                    sprint.FinnishDate = model.FinnishDate.Value;
                    this._repository.Add<Sprint>(sprint);
                    this._repository.SaveChanges();

                    foreach (var item in model.UserHistories)
                    {
                        if (item.Selected)
                        {
                            var backlog = new SprintBackLogUserHistory();
                            backlog.SprintId = sprint.SprintId;
                            backlog.UserHistoryId = item.UserHistoryId;
                            backlog.UserHistoryStatusId = item.UserHistoryStatusId;
                            backlog.ModifiedDate = DateTime.Now;
                            this._repository.Add<SprintBackLogUserHistory>(backlog);
                            this._repository.SaveChanges();
                        }
                    }

                    return RedirectToAction("Index");
                }
            }

            var project = this._repository.Single<Project>(model.ProjectId);
            var newModel = Mapper.Map<SprintCreateModel>(project); 
            return View(newModel);
        }

        //
        // GET: /Sprint/FinnishSprint/5

        public ActionResult FinnishSprint(int? sprintId)
        {
            var sprint = this._repository.Single<Sprint>(sprintId);
            if (sprint == null)
            {
                return HttpNotFound();
            }

            sprint.FinnishDate = DateTime.Today.AddDays(-1);
            this._repository.SaveChanges();

            return RedirectToAction("Index");
        }

        //
        // GET: /Sprint/EditTask

        public ActionResult EditTask(int? id)
        {
            var task = this._repository.Single<Task>(id);
            if (task == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<SprintTaskModel>(task);

            return View(model);
        }

        //
        // POST: /Sprint/EditTask

        [HttpPost]
        public ActionResult EditTask(SprintTaskModel model)
        {
            if (ModelState.IsValid)
            {
                var task = this._repository.Single<Task>(model.TaskId);
                task.UserHistoryId = model.UserHistoryId;
                task.TaskStatusId = (int)Config.TaskStatus.Pending;
                task.Description = model.Description;
                task.Estimation = model.Estimation;
                task.Name = model.Name;

                this._repository.SaveChanges();

                return RedirectToAction("ManageTasks", new { id = model.UserHistoryId, projectId = model.ProjectId });
            }

            return View(model); 
        }

        //
        // GET: /Sprint/UnassignTask/5

        public ActionResult UnassignTask(int? id, long? sprintId)
        {
            var task = this._repository.Single<Task>(id);
            if (task == null)
            {
                return HttpNotFound();
            }

            task.AssignedUserId = null;
            task.TaskStatusId = (int)Config.TaskStatus.Pending;

            var log = this._repository.Single<SprintBackLogUserHistory>(obj => obj.SprintId == sprintId && obj.UserHistoryId == task.UserHistoryId);
            var status = log.UserHistory.Task.Min(t => t.TaskStatusId);
            log.UserHistoryStatusId = status;
            log.ModifiedDate = DateTime.Now;

            this._repository.SaveChanges();

            return RedirectToAction("Index");
        }

        //
        // GET: /Sprint/AssignTask/5

        public ActionResult AssignTask(int? id, long? sprintId)
        {
            var task = this._repository.Single<Task>(id);
            var developers = this._repository.Query<User>(user => user.Permission.Any(pr => pr.ProfileId == (int)Config.Profile.Developer && pr.ProjectId == task.UserHistory.ProjectId));
            if (task == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<SprintAssignTaskModel>(task);
            model.SprintId = sprintId.GetValueOrDefault(0);
            model.Developers = new List<SprintUserModel>();
            foreach (var developer in developers)
            {
                model.Developers.Add(Mapper.Map<SprintUserModel>(developer));
            }

            return View(model);
        }

        //
        // POST: /Sprint/AssignTask/5   

        [HttpPost]
        public ActionResult AssignTask(SprintAssignTaskModel model)
        {
            if (ModelState.IsValid)
            {
                var task = this._repository.Single<Task>(model.TaskId);
                task.AssignedUserId = model.AssignedUserId;
                task.TaskStatusId = (int)Config.TaskStatus.InProcess;

                var log = this._repository.Single<SprintBackLogUserHistory>(obj => obj.SprintId == model.SprintId && obj.UserHistoryId == model.UserHistoryId);
                var status = log.UserHistory.Task.Min(t => t.TaskStatusId);
                log.UserHistoryStatusId = status;
                log.ModifiedDate = DateTime.Now;

                this._repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //
        // GET: /Sprint/AddWorkLog/5

        public ActionResult AddWorkLog(int? id, long? sprintId)
        {
            var task = this._repository.Single<Task>(id);
            if (task == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<SprintFinnishTaskModel>(task);
            model.SprintId = sprintId.Value;

            return View(model);
        }

        //
        // POST: /Sprint/AddWorkLog/5   

        [HttpPost]
        public ActionResult AddWorkLog(SprintFinnishTaskModel model)
        {
            if (ModelState.IsValid)
            {
                var workLog = new WorkLog();
                workLog.TaskId = model.TaskId;
                workLog.UserId = model.AssignedUserId.Value;
                workLog.SprintId = model.SprintId;
                workLog.TimeSpent = model.WorkLogItem;
                workLog.DateInserted = DateTime.Now;
                this._repository.Add<WorkLog>(workLog);

                this._repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }


        //
        // GET: /Sprint/FinnishTask/5

        public ActionResult FinnishTask(int? id, long? sprintId)
        {
            var task = this._repository.Single<Task>(id);
            if (task == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<SprintFinnishTaskModel>(task);
            model.SprintId = sprintId.Value;
            
            return View(model);
        }

        //
        // POST: /Sprint/FinnishTask/5   

        [HttpPost]
        public ActionResult FinnishTask(SprintFinnishTaskModel model)
        {
            if (ModelState.IsValid)
            {
                var workLog = new WorkLog();
                workLog.TaskId = model.TaskId;
                workLog.UserId = model.AssignedUserId.Value;
                workLog.SprintId = model.SprintId;
                workLog.TimeSpent = model.WorkLogItem;
                workLog.DateInserted = DateTime.Now;
                this._repository.Add<WorkLog>(workLog);

                var task = this._repository.Single<Task>(model.TaskId);
                task.TaskStatusId = (int)Config.TaskStatus.Finnished;

                var log = this._repository.Single<SprintBackLogUserHistory>(obj => obj.SprintId == model.SprintId && obj.UserHistoryId == model.UserHistoryId);
                var status = log.UserHistory.Task.Min(t => t.TaskStatusId);
                log.UserHistoryStatusId = status;
                log.ModifiedDate = DateTime.Now;

                this._repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //
        // GET: /Sprint/DeleteTask

        public ActionResult DeleteTask(int? id)
        {
            var task = this._repository.Single<Task>(id);
            var userHistoryId = task.UserHistoryId;
            var projectId = task.UserHistory.ProjectId;

            this._repository.Delete<Task>(id);
            this._repository.SaveChanges();

            return RedirectToAction("ManageTasks", new { id = userHistoryId, projectId = projectId });
        }


 
        protected override void Dispose(bool disposing)
        {
            this._repository.Dispose();
            base.Dispose(disposing);
        }
    }
}