﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;
using PagedList;
using DotNet.Highcharts;
using DotNet.Highcharts.Options;
using DotNet.Highcharts.Helpers;

namespace Scrum.Web.Controllers
{
    public class StatsController : Controller
    {
        private readonly IRepository _repository;
        private readonly IScrumMembershipProvider _membershipService;
        private readonly IScrumRoleProvider _authorizationService;

        public StatsController(IRepository repository, IScrumMembershipProvider membershipService, IScrumRoleProvider authorizationService)
        {
            this._repository = repository;
            this._membershipService = membershipService;
            this._authorizationService = authorizationService;
        }

        //
        // GET: /Stats/

        public ActionResult Index(int? projectId, int? page)
        {
            var selectedProjects = this._membershipService.GetSelectedProjects();
            var projects = this._repository.Query<Project>(proj => selectedProjects.Contains(proj.ProjectId));

            var model = Mapper.Map<IEnumerable<StatsListModel>>(projects);

            foreach (var item in model)
            {
                var pageNumber = 1;
                if (item.ProjectId == projectId)
                    pageNumber = page ?? 1;

                item.Sprints = item.Sprints.ToPagedList(pageNumber, Config.PageSize);
            }

            return View(model);
        }

        //
        // GET: /Stats/ShowStats/5

        public ActionResult ShowStats(long id = 0)
        {
            var sprint = this._repository.Single<Sprint>(id);
            if (sprint == null)
                return HttpNotFound();

            var model = Mapper.Map<StatsSprintModel>(sprint);
            string[] days = new string[10];

            for (int i = 0; i < 10; i++)
            {
                days[i] = model.BeginDate.AddDays(i * model.NumberOfElapsedDays / 9).ToString("dd/MM");
            }

            object[] idealData = new object[10];
            object[] realData = new object[10];
            var total = model.UserHistories.Sum(obj => obj.TotalTaskEstimation);
            for (int i = 0; i < 10; i++)
            {
                idealData[i] = decimal.Round(total - (i * total / 9), 2);
            }

            for (int i = 0; i < 10; i++)
            {
                var day = model.BeginDate.AddDays(i * model.NumberOfElapsedDays / 9).Date;
                if (day >= DateTime.Today) break;
                var done = this._repository.Query<WorkLog>(wl => wl.SprintId == model.SprintId && wl.DateInserted <= day).Sum(obj => (decimal?)obj.TimeSpent) ?? 0;
                realData[i] = total - done;
            }

            var ideal = new Series
                    {
                        Name="Ideal",
                        Data = new Data(idealData)
                    };
            

            var real = new Series
                    {
                        Name = "Real",
                        Data = new Data(realData)
                    };

            Highcharts chart = new DotNet.Highcharts.Highcharts("chart")
                .SetTitle(new Title() { Text = "Gràfica BurnDown" })
                .SetXAxis(new XAxis
                {
                    Title = new XAxisTitle() { Text = "Dies" },
                    Categories = days
                })
                .SetYAxis(new YAxis
                    {
                        Title = new YAxisTitle() { Text = "Temps" }
                    })
                .SetSeries(new Series[]
                {
                    ideal,
                    real
                });
            
            model.Chart = chart;

            return View(model);
        }

       
        protected override void Dispose(bool disposing)
        {
            this._repository.Dispose();
            base.Dispose(disposing);
        }
    }
}