﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;
using PagedList;

namespace Scrum.Web.Controllers
{
    [Authorize(Roles = "admin")]
    public class UserController : Controller
    {
        private readonly IRepository _repository;
        private readonly IScrumMembershipProvider _membershipService;
        private readonly IScrumRoleProvider _authorizationService;

        public UserController(IRepository repository, IScrumMembershipProvider membershipService, IScrumRoleProvider authorizationService)
        {
            this._repository = repository;
            this._membershipService = membershipService;
            this._authorizationService = authorizationService;
        }

        //
        // GET: /User/

        public ActionResult Index(int? page)
        {
            var users = _repository.All<User>();
            var pageNumber = page ?? 1;

            var model = Mapper.Map<IEnumerable<UserListModel>>(users).ToPagedList(pageNumber, Config.PageSize);

            return View(model);
        }

        //
        // GET: /User/Details/5

        public ActionResult Details(int id = 0)
        {
            var user = this._repository.Single<User>(id);
            if (user == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<UserDetailModel>(user);

            return View(model);
        }

        //
        // GET: /User/Create

        public ActionResult Create()
        {
            return View();
        }

        //
        // POST: /User/Create

        [HttpPost]
        public ActionResult Create(UserCreateModel model)
        {
            if (ModelState.IsValid)
            {
                User user = new User();
                user.Name = model.Name;
                user.Password = model.Password;
                user.Email = model.Email;
                user.IsAdmin = model.IsAdmin? (byte)1 : (byte)0;

                _repository.Add<User>(user);
                _repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //
        // GET: /User/Edit/5

        public ActionResult Edit(int id = 0)
        {
            var user = this._repository.Single<User>(id);
            if (user == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<UserEditModel>(user);

            return View(model);
        }

        //
        // POST: /User/Edit/5

        [HttpPost]
        public ActionResult Edit(UserEditModel model)
        {
            if (ModelState.IsValid)
            {
                var user = this._repository.Single<User>(model.UserId);
                user.Name = model.Name;
                user.Email = model.Email;
                user.IsAdmin = model.IsAdmin? (byte)1 : (byte)0;

                this._repository.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(model);
        }

        //
        // GET: /User/NewPassword/5

        public ActionResult NewPassword(int id = 0)
        {
            var user = this._repository.Single<User>(id);
            if (user == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<UserNewPasswordModel>(user);

            return View(model);
        }

        //
        // POST: /User/NewPassword/5

        [HttpPost]
        public ActionResult NewPassword(UserNewPasswordModel model)
        {
            if (ModelState.IsValid)
            {
                var user = this._repository.Single<User>(model.UserId);
                user.Password = model.Password;

                this._repository.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(model);
        }

        //
        // GET: /Project/ListRole/5

        public ActionResult ListRole(int id = 0)
        {
            var user = this._repository.Single<User>(id);
            if (user == null)
            {
                return HttpNotFound();
            }
            var projects = this._repository.All<Project>();

            var model = Mapper.Map<UserRoleListModel>(user);
            model.Owners = projects.Select(pr => new UserRoleListModelProject()
                {
                    ProjectId = pr.ProjectId,
                    ShortName = pr.ShortName,
                    Name = pr.Name,
                    HasAccess = pr.Permission.Any(per => per.ProfileId == 1 && per.UserId == user.UserId),
                    ProfileId = 1
                }).ToList();

            model.ScrumMasters = projects.Select(pr => new UserRoleListModelProject()
            {
                ProjectId = pr.ProjectId,
                ShortName = pr.ShortName,
                Name = pr.Name,
                HasAccess = pr.Permission.Any(per => per.ProfileId == 2 && per.UserId == user.UserId),
                ProfileId = 2
            }).ToList();
            model.Developers = projects.Select(pr => new UserRoleListModelProject()
            {
                ProjectId = pr.ProjectId,
                ShortName = pr.ShortName,
                Name = pr.Name,
                HasAccess = pr.Permission.Any(per => per.ProfileId == 3 && per.UserId == user.UserId),
                ProfileId = 3
            }).ToList();
            return View(model);
        }

        //
        // Post: /Project/ListRole/5

        [HttpPost]
        public ActionResult ListRole(UserRoleListModel model)
        {
            if (ModelState.IsValid)
            {
                this._repository.DeleteAll<Permission>(perm => perm.UserId == model.UserId);
                SavePermission(model.UserId.Value, model.Owners);
                SavePermission(model.UserId.Value, model.ScrumMasters);
                SavePermission(model.UserId.Value, model.Developers);
                this._repository.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(model);
        }

        private void SavePermission(int userId, List<UserRoleListModelProject> permissions)
        {
            foreach (var item in permissions)
            {
                if (item.HasAccess)
                {
                    this._repository.Add<Permission>(new Permission()
                    {
                        ProfileId = item.ProfileId,
                        UserId = userId,
                        ProjectId = item.ProjectId
                    });
                }
            }
        }

        protected override void Dispose(bool disposing)
        {
            this._repository.Dispose();
            base.Dispose(disposing);
        }
    }
}