﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;

namespace Scrum.Web.Models.DAL
{
    public class Repository : IRepository
    {
        private readonly DbContext _context;
        private readonly bool _autoCommit;

        public Repository() : this(new ScrumEntities(), true)
        {
        }

        private Repository(DbContext context, bool autoCommit = true)
        {
            if (context == null) throw new ArgumentNullException("context");

            _context = context;
            _autoCommit = autoCommit;
        }

        public void Add<TModel>(TModel instance) where TModel : class, IEntity
        {
            if (instance == null) throw new ArgumentNullException("instance");

            _context.Set<TModel>().Add(instance);

            SaveChangesWhenAutoCommit();
        }



        public void Add<TModel>(IEnumerable<TModel> instances) where TModel : class, IEntity
        {
            if (instances == null) throw new ArgumentNullException("instances");

            foreach (var instance in instances)
            {
                Add(instance);
            }
        }

        public IQueryable<TModel> All<TModel>() where TModel : class, IEntity
        {
            return Query<TModel>(x => true);
        }

        public void Delete<TModel>(object key) where TModel : class, IEntity
        {
            if (key == null) throw new ArgumentNullException("key");

            var entity = Single<TModel>(key);
            Delete(entity);
        }

        public void Delete<TModel>(TModel instance) where TModel : class, IEntity
        {
            if (instance == null) throw new ArgumentNullException("instance");

            _context.Set<TModel>().Remove(instance);
        }

        public void Delete<TModel>(System.Linq.Expressions.Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity
        {
            if (predicate == null) throw new ArgumentNullException("predicate");

            var entity = Single<TModel>(predicate);
            Delete(entity);
        }

        public void DeleteAll<TModel>(System.Linq.Expressions.Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity
        {
            if (predicate == null) throw new ArgumentNullException("predicate");

            var result = Query<TModel>(predicate);

            foreach (var item in result)
            {
                Delete(item);
            }
        }

        public TModel Single<TModel>(object key) where TModel : class, IEntity
        {
            if (key == null) throw new ArgumentNullException("key");

            return _context.Set<TModel>().Find(key);
        }

        public TModel Single<TModel>(System.Linq.Expressions.Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity
        {
            if (predicate == null) throw new ArgumentNullException("predicate");

            return _context.Set<TModel>().SingleOrDefault(predicate);
        }

        public IQueryable<TModel> Query<TModel>(System.Linq.Expressions.Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity
        {
            if (predicate == null) throw new ArgumentNullException("predicate");

            return _context.Set<TModel>().Where(predicate);
        }

        public void Dispose()
        {
            if (_context != null)
                _context.Dispose();
        }

        public int SaveChanges()
        {
            return SaveChangesContext();
        }

        private int SaveChangesContext()
        {
            if (_context == null) return 0;

            return _context.SaveChanges();
        }

        private int SaveChangesWhenAutoCommit()
        {
            if (_autoCommit) return SaveChangesContext();

            return 0;
        }
    }
}
